#include <stdio.h>
#include <stdlib.h>

#include <libxml/xmlmemory.h>
#include <libxml/parser.h>

#include "mem.h"
#include "xml_layout.h"
#include "xml_funcs.h"

static layout_item *xml_build_layouts (xmlNode *node, xmlNode *group);

static int layout_item_free (layout_item *layout)
	{
	if (!layout)
		return -1;
	
	if (layout->colour)
		mem_free (layout->colour);
	if (layout->bgcolour)
		mem_free (layout->bgcolour);
	
	if (layout->children)
		g_list_free (layout->children);
	
	mem_free (layout);
	
	return 0;
	}

static layout_item *layout_create_label (xmlNode *label, layout_item *parent)
	{
	layout_item *retval;
	char *tmp;
	
	retval = mem_alloc (sizeof (*retval));
	
	retval->type = LAYOUT_string;	
	retval->pos = LAYOUT_free;
	retval->children = NULL;
	retval->parent = parent;
	
	tmp = node_has_attribute (label, "size");
#warning "Should really be looking at parents etc... to get heights... or maybe later on?"
	retval->font_size = tmp ? atol (tmp) : 0;
	tmp = node_has_attribute (label, "font");	
	retval->font = tmp ? gnome_font_new (tmp, retval->font_size) : NULL;
	if (tmp && !retval->font)
		{
		msg_error ("Font not found: %s, using default", tmp);
#warning "Not assigning default font correctly"
		if (retval->font_size)
			retval->font = gnome_font_new ("Helvetica", retval->font_size);
		else
			retval->font = gnome_font_new ("Helvetica", 14);
	}
	else
	{
#warning "should be assigning parents font to this child here"
		retval->font = gnome_font_new ("Helvetica", 14);
	}
	
	return retval;
	}
	
static layout_position convert_string_to_pos (char *string)
	{
	if (!string)
		return LAYOUT_free;
	if (strcmp (string, "right") == 0)
		return LAYOUT_right;
	if (strcmp (string, "left") == 0)
		return LAYOUT_left;
	if (strcmp (string, "above") == 0)
		return LAYOUT_above;
	if (strcmp (string, "below") == 0)
		return LAYOUT_below;
	
	msg_error ("Invalid position: %s", string);
	return LAYOUT_free;
	}
	
static layout_item *layout_create_node (xmlNode *node, layout_item *parent)
	{
	layout_item *retval = NULL;
	
	if (!node)
		return NULL;

	if (strcmp (node->name, "label") == 0)
		retval = layout_create_label (node, parent);
	if (strcmp (node->name, "group") == 0)
		retval = xml_build_layouts (node->childs, node);
		
	if (retval)	
		{
		retval->name = node_has_attribute (node, "name");	
		retval->data = node_has_attribute (node, "data");
		retval->pos = convert_string_to_pos (node_has_attribute (node, "pos"));
		
		retval->node = node;
		retval->parent = parent;
		msg_debug ("adding node: %s, %s", retval->name, retval->data);
		}
	
	return retval;
	}

static layout_item *xml_build_layouts (xmlNode *node, xmlNode *group)
	{
	layout_item *retval = NULL;
	xmlNode *cur;
	layout_item *item;

	if (!node)
		return NULL;
	
	retval = mem_alloc (sizeof (*retval));
	retval->type = LAYOUT_group;
	retval->pos = LAYOUT_free;
	retval->name = NULL;
	retval->data = NULL;
	retval->children = NULL;
	
	for (cur = node; cur != NULL; cur = cur->next)
		{
		item = layout_create_node (cur, retval);
		if (item)
			{
			item->group = group;
			retval->children = g_list_append (retval->children, item);
			}
		else
			msg_error ("Unknown layout item: %s", cur->name);
		}
		
	return retval;
}
	
static report_item *xml_build_group (xmlNode *group)
	{
	report_item *retval = NULL;
	xmlNode *node;
	
	if (!group)
		return NULL;

	retval = mem_alloc (sizeof (*retval));
	retval->type = REPORT_group;
	retval->body = retval->header = retval->footer = NULL;
	
	for (node = group->childs; node != NULL; node = node->next)
		{
		if (strcmp (node->name, "header") == 0)
			retval->header = xml_build_layouts (node->childs, group);
		else if (strcmp (node->name, "body") == 0)
			retval->body = xml_build_layouts (node->childs, group);
		else if (strcmp (node->name, "footer") == 0)
			retval->footer = xml_build_layouts (node->childs, group);
		else
			msg_error ("Unknown group item: %s", node->name);
		}
	
	return retval;	
	}
	
static GList *xml_build_report (xmlNode *report)
	{
	GList *retval = NULL;
	xmlNode *cur;
	
	if (strcmp (report->name, "report") != 0)
		{
		msg_error ("Not a report: %s", report->name);
		return NULL;
		}
		
	for (cur = report->childs; cur != NULL; cur = cur->next)
		{
		if (strcmp (cur->name, "group") == 0)
			retval = g_list_append (retval, xml_build_group (cur));
		else
			msg_error ("Unknown report item: %s", cur->name);
		}
		
	return retval;
	}
	
static void xml_display_layout_item (layout_item *item, int indent)
	{
	GList *cur;
	
	msg_display ("%*slayout item [%d] %s (%s)", indent, "", item->type, item->name, item->data);
	
	for (cur = item->children; cur != NULL; cur = cur->next)
		xml_display_layout_item (cur->data, indent + 2);
	}
	
static void xml_display_report_item (report_item *report, int indent)
	{
	msg_display ("%*sreport item %d", indent * 2, "", report->type);

	indent+=2;
	
	if (report->header)
		{
		msg_display ("%*sreport header", indent, "");
		xml_display_layout_item (report->header, indent + 2);
		}
		
	if (report->body)
		{
		msg_display ("%*sreport body", indent, "");
		xml_display_layout_item (report->body, indent + 2);
		}
		
	if (report->footer)
		{
		msg_display ("%*sreport footer", indent, "");
		xml_display_layout_item (report->footer, indent + 2);
		}
	}
	
static void xml_display_layout (GList *layouts)
	{
	GList *cur;
	
	for (cur = layouts; cur != NULL; cur = cur->next)
		xml_display_report_item (cur->data, 0);
	}
	
GList *xml_build_file (xmlDoc *doc)
	{
	xmlNode *node;
	GList *retval = NULL;
	
	if (!doc)
		return NULL;
	
	for (node = doc->root; node != NULL; node = node->next)
		{
		if (strcmp (node->name, "report") == 0)
			retval = g_list_concat (retval, xml_build_report (node));
		}
		
	xml_display_layout (retval);
		
	return retval;
	}
